@@ -1,5 +1,5 @@
 /*  Lzip - LZMA lossless data compressor
-    Copyright (C) 2008-2016 Antonio Diaz Diaz.
+    Copyright (C) 2008-2019 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,20 +38,25 @@
 #include <unistd.h>
 #include <utime.h>
 #include <sys/stat.h>
-#if defined(__MSVCRT__)
+#if defined(__MSVCRT__) || defined(__OS2__) || defined(__DJGPP__)
 #include <io.h>
+#if defined(__MSVCRT__)
 #define fchmod(x,y) 0
 #define fchown(x,y,z) 0
 #define strtoull std::strtoul
 #define SIGHUP SIGTERM
 #define S_ISSOCK(x) 0
+#ifndef S_IRGRP
 #define S_IRGRP 0
 #define S_IWGRP 0
 #define S_IROTH 0
 #define S_IWOTH 0
 #endif
-#if defined(__OS2__)
-#include <io.h>
+#endif
+#if defined(__DJGPP__)
+#define S_ISSOCK(x) 0
+#define S_ISVTX 0
+#endif
 #endif
 
 #include "arg_parser.h"
@@ -73,12 +78,11 @@ int verbosity = 0;
 
 namespace {
 
-const char * const Program_name = "Lzip";
 const char * const program_name = "lzip";
-const char * const program_year = "2016";
+const char * const program_year = "2019";
 const char * invocation_name = 0;
 
-struct { const char * from; const char * to; } const known_extensions[] = {
+const struct { const char * from; const char * to; } known_extensions[] = {
   { ".lz",  ""     },
   { ".tlz", ".tar" },
   { 0,      0      } };
@@ -89,8 +93,10 @@ struct Lzma_options
   int match_len_limit;		// 5 .. 273
   };
 
-enum Mode { m_compress, m_decompress, m_test };
+enum Mode { m_compress, m_decompress, m_list, m_test };
 
+/* Variables used in signal handler context.
+   They are not declared volatile because the handler never returns. */
 std::string output_filename;
 int outfd = -1;
 bool delete_output_on_interrupt = false;
@@ -98,8 +104,14 @@ bool delete_output_on_interrupt = false;
 
 void show_help()
   {
-  std::printf( "%s - LZMA lossless data compressor.\n", Program_name );
-  std::printf( "\nUsage: %s [options] [files]\n", invocation_name );
+  std::printf( "Lzip is a lossless data compressor with a user interface similar to the\n"
+               "one of gzip or bzip2. Lzip can compress about as fast as gzip (lzip -0)\n"
+               "or compress most files more than bzip2 (lzip -9). Decompression speed is\n"
+               "intermediate between gzip and bzip2. Lzip is better than gzip and bzip2\n"
+               "from a data recovery perspective. Lzip has been designed, written and\n"
+               "tested with great care to replace gzip and bzip2 as the standard\n"
+               "general-purpose compressed format for unix-like systems.\n"
+               "\nUsage: %s [options] [files]\n", invocation_name );
   std::printf( "\nOptions:\n"
                "  -h, --help                     display this help and exit\n"
                "  -V, --version                  output version information and exit\n"
@@ -110,6 +122,7 @@ void show_help()
                "  -f, --force                    overwrite existing output files\n"
                "  -F, --recompress               force re-compression of compressed files\n"
                "  -k, --keep                     keep (don't delete) input files\n"
+               "  -l, --list                     print (un)compressed file sizes\n"
                "  -m, --match-length=<bytes>     set match length limit in bytes [36]\n"
                "  -o, --output=<file>            if reading standard input, write to <file>\n"
                "  -q, --quiet                    suppress all messages\n"
@@ -120,6 +133,7 @@ void show_help()
                "  -0 .. -9                       set compression level [default 6]\n"
                "      --fast                     alias for -0\n"
                "      --best                     alias for -9\n"
+               "      --loose-trailing           allow trailing data seeming corrupt header\n"
                "If no file names are given, or if a file is '-', lzip compresses or\n"
                "decompresses from standard input to standard output.\n"
                "Numbers may be followed by a multiplier: k = kB = 10^3 = 1000,\n"
@@ -148,27 +162,58 @@ void show_version()
                "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
+} // end namespace
 
-void show_header( const unsigned dictionary_size )
+void Pretty_print::operator()( const char * const msg ) const
   {
-  if( verbosity >= 3 )
+  if( verbosity >= 0 )
     {
-    const char * const prefix[8] =
-      { "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi", "Yi" };
-    enum { factor = 1024 };
-    const char * p = "";
-    const char * np = "  ";
-    unsigned num = dictionary_size;
-    bool exact = ( num % factor == 0 );
-
-    for( int i = 0; i < 8 && ( num > 9999 || ( exact && num >= factor ) ); ++i )
-      { num /= factor; if( num % factor != 0 ) exact = false;
-        p = prefix[i]; np = ""; }
-    std::fprintf( stderr, "dictionary size %s%4u %sB.  ", np, num, p );
+    if( first_post )
+      {
+      first_post = false;
+      std::fputs( padded_name.c_str(), stderr );
+      if( !msg ) std::fflush( stderr );
+      }
+    if( msg ) std::fprintf( stderr, "%s\n", msg );
     }
   }
 
 
+const char * bad_version( const unsigned version )
+  {
+  static char buf[80];
+  snprintf( buf, sizeof buf, "Version %u member format not supported.",
+            version );
+  return buf;
+  }
+
+
+const char * format_ds( const unsigned dictionary_size )
+  {
+  enum { bufsize = 16, factor = 1024 };
+  static char buf[bufsize];
+  const char * const prefix[8] =
+    { "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi", "Yi" };
+  const char * p = "";
+  const char * np = "  ";
+  unsigned num = dictionary_size;
+  bool exact = ( num % factor == 0 );
+
+  for( int i = 0; i < 8 && ( num > 9999 || ( exact && num >= factor ) ); ++i )
+    { num /= factor; if( num % factor != 0 ) exact = false;
+      p = prefix[i]; np = ""; }
+  snprintf( buf, bufsize, "%s%4u %sB", np, num, p );
+  return buf;
+  }
+
+
+void show_header( const unsigned dictionary_size )
+  {
+  std::fprintf( stderr, "dictionary %s, ", format_ds( dictionary_size ) );
+  }
+
+namespace {
+
 unsigned long long getnum( const char * const ptr,
                            const unsigned long long llimit,
                            const unsigned long long ulimit )
@@ -184,7 +229,7 @@ unsigned long long getnum( const char * const ptr,
 
   if( !errno && tail[0] )
     {
-    const int factor = ( tail[1] == 'i' ) ? 1024 : 1000;
+    const unsigned factor = ( tail[1] == 'i' ) ? 1024 : 1000;
     int exponent = 0;				// 0 = bad multiplier
     switch( tail[0] )
       {
@@ -222,88 +267,57 @@ unsigned long long getnum( const char * const ptr,
 int get_dict_size( const char * const arg )
   {
   char * tail;
-  const int bits = std::strtol( arg, &tail, 0 );
+  const long bits = std::strtol( arg, &tail, 0 );
   if( bits >= min_dictionary_bits &&
       bits <= max_dictionary_bits && *tail == 0 )
-    return ( 1 << bits );
+    return 1 << bits;
   return getnum( arg, min_dictionary_size, max_dictionary_size );
   }
 
 
+void set_mode( Mode & program_mode, const Mode new_mode )
+  {
+  if( program_mode != m_compress && program_mode != new_mode )
+    {
+    show_error( "Only one operation can be specified.", 0, true );
+    std::exit( 1 );
+    }
+  program_mode = new_mode;
+  }
+
+
 int extension_index( const std::string & name )
   {
-  for( int i = 0; known_extensions[i].from; ++i )
+  for( int eindex = 0; known_extensions[eindex].from; ++eindex )
     {
-    const std::string ext( known_extensions[i].from );
+    const std::string ext( known_extensions[eindex].from );
     if( name.size() > ext.size() &&
         name.compare( name.size() - ext.size(), ext.size(), ext ) == 0 )
-      return i;
+      return eindex;
     }
   return -1;
   }
 
 
-int open_instream( const char * const name, struct stat * const in_statsp,
-                   const Mode program_mode, const int eindex,
-                   const bool recompress, const bool to_stdout )
-  {
-  int infd = -1;
-  if( program_mode == m_compress && !recompress && eindex >= 0 )
-    {
-    if( verbosity >= 0 )
-      std::fprintf( stderr, "%s: Input file '%s' already has '%s' suffix.\n",
-                    program_name, name, known_extensions[eindex].from );
-    }
-  else
-    {
-    infd = open( name, O_RDONLY | O_BINARY );
-    if( infd < 0 )
-      {
-      if( verbosity >= 0 )
-        std::fprintf( stderr, "%s: Can't open input file '%s': %s\n",
-                      program_name, name, std::strerror( errno ) );
-      }
-    else
-      {
-      const int i = fstat( infd, in_statsp );
-      const mode_t mode = in_statsp->st_mode;
-      const bool can_read = ( i == 0 &&
-                              ( S_ISBLK( mode ) || S_ISCHR( mode ) ||
-                                S_ISFIFO( mode ) || S_ISSOCK( mode ) ) );
-      const bool no_ofile = ( to_stdout || program_mode == m_test );
-      if( i != 0 || ( !S_ISREG( mode ) && ( !can_read || !no_ofile ) ) )
-        {
-        if( verbosity >= 0 )
-          std::fprintf( stderr, "%s: Input file '%s' is not a regular file%s.\n",
-                        program_name, name,
-                        ( can_read && !no_ofile ) ?
-                        ",\n      and '--stdout' was not specified" : "" );
-        close( infd );
-        infd = -1;
-        }
-      }
-    }
-  return infd;
-  }
-
-
-void set_c_outname( const std::string & name, const bool multifile )
+void set_c_outname( const std::string & name, const bool force_ext,
+                    const bool multifile )
   {
   output_filename = name;
   if( multifile ) output_filename += "00001";
-  output_filename += known_extensions[0].from;
+  if( force_ext || multifile || extension_index( output_filename ) < 0 )
+    output_filename += known_extensions[0].from;
   }
 
 
-void set_d_outname( const std::string & name, const int i )
+void set_d_outname( const std::string & name, const int eindex )
   {
-  if( i >= 0 )
+  if( eindex >= 0 )
     {
-    const std::string from( known_extensions[i].from );
+    const std::string from( known_extensions[eindex].from );
     if( name.size() > from.size() )
       {
       output_filename.assign( name, 0, name.size() - from.size() );
-      output_filename += known_extensions[i].to;
+      output_filename += known_extensions[eindex].to;
       return;
       }
     }
@@ -313,6 +327,52 @@ void set_d_outname( const std::string & name, const int i )
                   program_name, name.c_str(), output_filename.c_str() );
   }
 
+} // end namespace
+
+int open_instream( const char * const name, struct stat * const in_statsp,
+                   const bool no_ofile, const bool reg_only )
+  {
+  int infd = open( name, O_RDONLY | O_BINARY );
+  if( infd < 0 )
+    show_file_error( name, "Can't open input file", errno );
+  else
+    {
+    const int i = fstat( infd, in_statsp );
+    const mode_t mode = in_statsp->st_mode;
+    const bool can_read = ( i == 0 && !reg_only &&
+                            ( S_ISBLK( mode ) || S_ISCHR( mode ) ||
+                              S_ISFIFO( mode ) || S_ISSOCK( mode ) ) );
+    if( i != 0 || ( !S_ISREG( mode ) && ( !can_read || !no_ofile ) ) )
+      {
+      if( verbosity >= 0 )
+        std::fprintf( stderr, "%s: Input file '%s' is not a regular file%s.\n",
+                      program_name, name,
+                      ( can_read && !no_ofile ) ?
+                      ",\n      and '--stdout' was not specified" : "" );
+      close( infd );
+      infd = -1;
+      }
+    }
+  return infd;
+  }
+
+namespace {
+
+int open_instream2( const char * const name, struct stat * const in_statsp,
+                    const Mode program_mode, const int eindex,
+                    const bool recompress, const bool to_stdout )
+  {
+  if( program_mode == m_compress && !recompress && eindex >= 0 )
+    {
+    if( verbosity >= 0 )
+      std::fprintf( stderr, "%s: Input file '%s' already has '%s' suffix.\n",
+                    program_name, name, known_extensions[eindex].from );
+    return -1;
+    }
+  const bool no_ofile = ( to_stdout || program_mode == m_test );
+  return open_instream( name, in_statsp, no_ofile, false );
+  }
+
 
 bool open_outstream( const bool force, const bool from_stdin )
   {
@@ -337,7 +397,8 @@ bool open_outstream( const bool force, const bool from_stdin )
   }
 
 
-bool check_tty( const int infd, const Mode program_mode )
+bool check_tty( const char * const input_filename, const int infd,
+                const Mode program_mode )
   {
   if( program_mode == m_compress && isatty( outfd ) )
     {
@@ -347,15 +408,25 @@ bool check_tty( const int infd, const Mode program_mode )
   if( ( program_mode == m_decompress || program_mode == m_test ) &&
       isatty( infd ) )
     {
-    show_error( "I won't read compressed data from a terminal.", 0, true );
+    show_file_error( input_filename,
+                     "I won't read compressed data from a terminal." );
     return false;
     }
   return true;
   }
 
 
+void set_signals( void (*action)(int) )
+  {
+  std::signal( SIGHUP, action );
+  std::signal( SIGINT, action );
+  std::signal( SIGTERM, action );
+  }
+
+
 void cleanup_and_fail( const int retval )
   {
+  set_signals( SIG_IGN );			// ignore signals
   if( delete_output_on_interrupt )
     {
     delete_output_on_interrupt = false;
@@ -370,6 +441,13 @@ void cleanup_and_fail( const int retval )
   }
 
 
+extern "C" void signal_handler( int )
+  {
+  show_error( "Control-C or similar caught, quitting." );
+  cleanup_and_fail( 1 );
+  }
+
+
      // Set permissions, owner and times.
 void close_and_set_permissions( const struct stat * const in_statsp )
   {
@@ -406,9 +484,10 @@ void close_and_set_permissions( const struct stat * const in_statsp )
 
 bool next_filename()
   {
+  const unsigned name_len = output_filename.size();
   const unsigned ext_len = std::strlen( known_extensions[0].from );
-  if( output_filename.size() >= ext_len + 5 )		// "*00001.lz"
-    for( int i = output_filename.size() - ext_len - 1, j = 0; j < 5; --i, ++j )
+  if( name_len >= ext_len + 5 )				// "*00001.lz"
+    for( int i = name_len - ext_len - 1, j = 0; j < 5; --i, ++j )
       {
       if( output_filename[i] < '9' ) { ++output_filename[i]; return true; }
       else output_filename[i] = '0';
@@ -417,13 +496,12 @@ bool next_filename()
   }
 
 
-int compress( const unsigned long long member_size,
+int compress( const unsigned long long cfile_size,
+              const unsigned long long member_size,
               const unsigned long long volume_size, const int infd,
               const Lzma_options & encoder_options, const Pretty_print & pp,
               const struct stat * const in_statsp, const bool zero )
   {
-  const unsigned long long cfile_size =
-    (in_statsp && S_ISREG( in_statsp->st_mode )) ? in_statsp->st_size / 100 : 0;
   int retval = 0;
   LZ_encoder_base * encoder = 0;		// polymorphic encoder
   if( verbosity >= 1 ) pp();
@@ -433,7 +511,7 @@ int compress( const unsigned long long member_size,
       encoder = new FLZ_encoder( infd, outfd );
     else
       {
-      File_header header;
+      Lzip_header header;
       if( header.dictionary_size( encoder_options.dictionary_size ) &&
           encoder_options.match_len_limit >= min_match_len_limit &&
           encoder_options.match_len_limit <= max_match_len )
@@ -447,7 +525,7 @@ int compress( const unsigned long long member_size,
       {
       const unsigned long long size = ( volume_size > 0 ) ?
         std::min( member_size, volume_size - partial_volume_size ) : member_size;
-      show_progress( in_size, encoder, &pp, cfile_size );	// init
+      show_cprogress( cfile_size, in_size, encoder, &pp );	// init
       if( !encoder->encode_member( size ) )
         { pp( "Encoder error." ); retval = 1; break; }
       in_size += encoder->data_position();
@@ -476,69 +554,62 @@ int compress( const unsigned long long member_size,
       if( in_size == 0 || out_size == 0 )
         std::fputs( " no data compressed.\n", stderr );
       else
-        std::fprintf( stderr, "%6.3f:1, %6.3f bits/byte, "
-                              "%5.2f%% saved, %llu in, %llu out.\n",
+        std::fprintf( stderr, "%6.3f:1, %5.2f%% ratio, %5.2f%% saved, "
+                              "%llu in, %llu out.\n",
                       (double)in_size / out_size,
-                      ( 8.0 * out_size ) / in_size,
-                      100.0 * ( 1.0 - ( (double)out_size / in_size ) ),
+                      ( 100.0 * out_size ) / in_size,
+                      100.0 - ( ( 100.0 * out_size ) / in_size ),
                       in_size, out_size );
       }
     }
-  catch( std::bad_alloc )
+  catch( std::bad_alloc & )
     {
     pp( "Not enough memory. Try a smaller dictionary size." );
     retval = 1;
     }
-  catch( Error e ) { pp(); show_error( e.msg, errno ); retval = 1; }
+  catch( Error & e ) { pp(); show_error( e.msg, errno ); retval = 1; }
   delete encoder;
   return retval;
   }
 
 
-unsigned char xdigit( const int value )
+unsigned char xdigit( const unsigned value )
   {
-  if( value >= 0 && value <= 9 ) return '0' + value;
-  if( value >= 10 && value <= 15 ) return 'A' + value - 10;
+  if( value <= 9 ) return '0' + value;
+  if( value <= 15 ) return 'A' + value - 10;
   return 0;
   }
 
 
 bool show_trailing_data( const uint8_t * const data, const int size,
                          const Pretty_print & pp, const bool all,
-                         const bool ignore_trailing )
+                         const int ignore_trailing )	// -1 = show
   {
-  if( verbosity >= 4 || !ignore_trailing )
+  if( verbosity >= 4 || ignore_trailing <= 0 )
     {
     std::string msg;
     if( !all ) msg = "first bytes of ";
     msg += "trailing data = ";
-    bool text = true;
     for( int i = 0; i < size; ++i )
-      if( !std::isprint( data[i] ) ) { text = false; break; }
-    if( text )
       {
-      msg += '\'';
-      msg.append( (const char *)data, size );
-      msg += '\'';
-      }
-    else
-      {
-      for( int i = 0; i < size; ++i )
-        {
-        if( i > 0 ) msg += ' ';
-        msg += xdigit( data[i] >> 4 );
-        msg += xdigit( data[i] & 0x0F );
-        }
+      msg += xdigit( data[i] >> 4 );
+      msg += xdigit( data[i] & 0x0F );
+      msg += ' ';
       }
+    msg += '\'';
+    for( int i = 0; i < size; ++i )
+      { if( std::isprint( data[i] ) ) msg += data[i]; else msg += '.'; }
+    msg += '\'';
     pp( msg.c_str() );
-    if( !ignore_trailing ) show_error( "Trailing data not allowed." );
+    if( ignore_trailing == 0 ) show_file_error( pp.name(), trailing_msg );
     }
-  return ignore_trailing;
+  return ( ignore_trailing > 0 );
   }
 
 
-int decompress( const int infd, const Pretty_print & pp,
-                const bool ignore_trailing, const bool testing )
+int decompress( const unsigned long long cfile_size, const int infd,
+                const Pretty_print & pp, const bool ignore_trailing,
+                const bool loose_trailing, const bool testing )
   {
   int retval = 0;
 
@@ -547,13 +618,18 @@ int decompress( const int infd, const Pretty_print & pp,
     Range_decoder rdec( infd );
     for( bool first_member = true; ; first_member = false )
       {
-      File_header header;
+      Lzip_header header;
       rdec.reset_member_position();
-      const int size = rdec.read_data( header.data, File_header::size );
+      const int size = rdec.read_data( header.data, Lzip_header::size );
       if( rdec.finished() )			// End Of File
         {
-        if( first_member || header.verify_prefix( size ) )
-          { pp( "File ends unexpectedly at member header." ); retval = 2; }
+        if( first_member )
+          { show_file_error( pp.name(), "File ends unexpectedly at member header." );
+            retval = 2; }
+        else if( header.verify_prefix( size ) )
+          { pp( "Truncated header in multimember file." );
+            show_trailing_data( header.data, size, pp, true, -1 );
+            retval = 2; }
         else if( size > 0 && !show_trailing_data( header.data, size, pp,
                                                   true, ignore_trailing ) )
           retval = 2;
@@ -562,27 +638,25 @@ int decompress( const int infd, const Pretty_print & pp,
       if( !header.verify_magic() )
         {
         if( first_member )
-          { pp( "Bad magic number (file not in lzip format)." ); retval = 2; }
+          { show_file_error( pp.name(), bad_magic_msg ); retval = 2; }
+        else if( !loose_trailing && header.verify_corrupt() )
+          { pp( corrupt_mm_msg );
+            show_trailing_data( header.data, size, pp, false, -1 );
+            retval = 2; }
         else if( !show_trailing_data( header.data, size, pp, false, ignore_trailing ) )
           retval = 2;
         break;
         }
       if( !header.verify_version() )
-        {
-        if( verbosity >= 0 )
-          { pp();
-            std::fprintf( stderr, "Version %d member format not supported.\n",
-                          header.version() ); }
-        retval = 2; break;
-        }
+        { pp( bad_version( header.version() ) ); retval = 2; break; }
       const unsigned dictionary_size = header.dictionary_size();
       if( !isvalid_ds( dictionary_size ) )
-        { pp( "Invalid dictionary size in member header." ); retval = 2; break; }
+        { pp( bad_dict_msg ); retval = 2; break; }
 
-      if( verbosity >= 2 || ( verbosity == 1 && first_member ) )
-        { pp(); show_header( dictionary_size ); }
+      if( verbosity >= 2 || ( verbosity == 1 && first_member ) ) pp();
 
       LZ_decoder decoder( rdec, dictionary_size, outfd );
+      show_dprogress( cfile_size, partial_file_pos, &rdec, &pp );	// init
       const int result = decoder.decode_member( pp );
       partial_file_pos += rdec.member_position();
       if( result != 0 )
@@ -600,28 +674,13 @@ int decompress( const int infd, const Pretty_print & pp,
         { std::fputs( testing ? "ok\n" : "done\n", stderr ); pp.reset(); }
       }
     }
-  catch( std::bad_alloc ) { pp( "Not enough memory." ); retval = 1; }
-  catch( Error e ) { pp(); show_error( e.msg, errno ); retval = 1; }
+  catch( std::bad_alloc & ) { pp( "Not enough memory." ); retval = 1; }
+  catch( Error & e ) { pp(); show_error( e.msg, errno ); retval = 1; }
   if( verbosity == 1 && retval == 0 )
     std::fputs( testing ? "ok\n" : "done\n", stderr );
   return retval;
   }
 
-
-extern "C" void signal_handler( int )
-  {
-  show_error( "Control-C or similar caught, quitting." );
-  cleanup_and_fail( 1 );
-  }
-
-
-void set_signals()
-  {
-  std::signal( SIGHUP, signal_handler );
-  std::signal( SIGINT, signal_handler );
-  std::signal( SIGTERM, signal_handler );
-  }
-
 } // end namespace
 
 
@@ -629,17 +688,25 @@ void show_error( const char * const msg, const int errcode, const bool help )
   {
   if( verbosity < 0 ) return;
   if( msg && msg[0] )
-    {
-    std::fprintf( stderr, "%s: %s", program_name, msg );
-    if( errcode > 0 ) std::fprintf( stderr, ": %s", std::strerror( errcode ) );
-    std::fputc( '\n', stderr );
-    }
+    std::fprintf( stderr, "%s: %s%s%s\n", program_name, msg,
+                  ( errcode > 0 ) ? ": " : "",
+                  ( errcode > 0 ) ? std::strerror( errcode ) : "" );
   if( help )
     std::fprintf( stderr, "Try '%s --help' for more information.\n",
                   invocation_name );
   }
 
 
+void show_file_error( const char * const filename, const char * const msg,
+                      const int errcode )
+  {
+  if( verbosity >= 0 )
+    std::fprintf( stderr, "%s: %s: %s%s%s\n", program_name, filename, msg,
+                  ( errcode > 0 ) ? ": " : "",
+                  ( errcode > 0 ) ? std::strerror( errcode ) : "" );
+  }
+
+
 void internal_error( const char * const msg )
   {
   if( verbosity >= 0 )
@@ -648,25 +715,61 @@ void internal_error( const char * const msg )
   }
 
 
-void show_progress( const unsigned long long partial_size,
-                    const Matchfinder_base * const m,
-                    const Pretty_print * const p,
-                    const unsigned long long cfile_size )
+void show_cprogress( const unsigned long long cfile_size,
+                     const unsigned long long partial_size,
+                     const Matchfinder_base * const m,
+                     const Pretty_print * const p )
   {
   static unsigned long long csize = 0;		// file_size / 100
   static unsigned long long psize = 0;
   static const Matchfinder_base * mb = 0;
   static const Pretty_print * pp = 0;
+  static bool enabled = true;
 
-  if( verbosity < 2 ) return;
-  if( m )					// initialize static vars
-    { csize = cfile_size; psize = partial_size; mb = m; pp = p; }
+  if( !enabled ) return;
+  if( p )					// initialize static vars
+    {
+    if( verbosity < 2 || !isatty( STDERR_FILENO ) ) { enabled = false; return; }
+    csize = cfile_size; psize = partial_size; mb = m; pp = p;
+    }
   if( mb && pp )
     {
     const unsigned long long pos = psize + mb->data_position();
     if( csize > 0 )
-      std::fprintf( stderr, "%4llu%%", pos / csize );
-    std::fprintf( stderr, "  %.1f MB\r", pos / 1000000.0 );
+      std::fprintf( stderr, "%4llu%%  %.1f MB\r", pos / csize, pos / 1000000.0 );
+    else
+      std::fprintf( stderr, "  %.1f MB\r", pos / 1000000.0 );
+    pp->reset(); (*pp)();			// restore cursor position
+    }
+  }
+
+
+void show_dprogress( const unsigned long long cfile_size,
+                     const unsigned long long partial_size,
+                     const Range_decoder * const d,
+                     const Pretty_print * const p )
+  {
+  static unsigned long long csize = 0;		// file_size / 100
+  static unsigned long long psize = 0;
+  static const Range_decoder * rdec = 0;
+  static const Pretty_print * pp = 0;
+  static int counter = 0;
+  static bool enabled = true;
+
+  if( !enabled ) return;
+  if( p )					// initialize static vars
+    {
+    if( verbosity < 2 || !isatty( STDERR_FILENO ) ) { enabled = false; return; }
+    csize = cfile_size; psize = partial_size; rdec = d; pp = p; counter = 0;
+    }
+  if( rdec && pp && --counter <= 0 )
+    {
+    const unsigned long long pos = psize + rdec->member_position();
+    counter = 7;		// update display every 114688 bytes
+    if( csize > 0 )
+      std::fprintf( stderr, "%4llu%%  %.1f MB\r", pos / csize, pos / 1000000.0 );
+    else
+      std::fprintf( stderr, "  %.1f MB\r", pos / 1000000.0 );
     pp->reset(); (*pp)();			// restore cursor position
     }
   }
@@ -678,7 +781,7 @@ int main( const int argc, const char * const argv[] )
      to the corresponding LZMA compression modes. */
   const Lzma_options option_mapping[] =
     {
-    { 1 << 16,  16 },		// -0 entry values not used
+    { 1 << 16,  16 },		// -0
     { 1 << 20,   5 },		// -1
     { 3 << 19,   6 },		// -2
     { 1 << 21,   8 },		// -3
@@ -693,49 +796,51 @@ int main( const int argc, const char * const argv[] )
   const unsigned long long max_volume_size = 0x4000000000000000ULL;
   unsigned long long member_size = max_member_size;
   unsigned long long volume_size = 0;
-  std::string input_filename;
   std::string default_output_filename;
   std::vector< std::string > filenames;
-  int infd = -1;
   Mode program_mode = m_compress;
   bool force = false;
   bool ignore_trailing = true;
   bool keep_input_files = false;
+  bool loose_trailing = false;
   bool recompress = false;
   bool to_stdout = false;
   bool zero = false;
   invocation_name = argv[0];
 
+  enum { opt_lt = 256 };
   const Arg_parser::Option options[] =
     {
-    { '0', "fast",            Arg_parser::no  },
-    { '1',  0,                Arg_parser::no  },
-    { '2',  0,                Arg_parser::no  },
-    { '3',  0,                Arg_parser::no  },
-    { '4',  0,                Arg_parser::no  },
-    { '5',  0,                Arg_parser::no  },
-    { '6',  0,                Arg_parser::no  },
-    { '7',  0,                Arg_parser::no  },
-    { '8',  0,                Arg_parser::no  },
-    { '9', "best",            Arg_parser::no  },
-    { 'a', "trailing-error",  Arg_parser::no  },
-    { 'b', "member-size",     Arg_parser::yes },
-    { 'c', "stdout",          Arg_parser::no  },
-    { 'd', "decompress",      Arg_parser::no  },
-    { 'f', "force",           Arg_parser::no  },
-    { 'F', "recompress",      Arg_parser::no  },
-    { 'h', "help",            Arg_parser::no  },
-    { 'k', "keep",            Arg_parser::no  },
-    { 'm', "match-length",    Arg_parser::yes },
-    { 'n', "threads",         Arg_parser::yes },
-    { 'o', "output",          Arg_parser::yes },
-    { 'q', "quiet",           Arg_parser::no  },
-    { 's', "dictionary-size", Arg_parser::yes },
-    { 'S', "volume-size",     Arg_parser::yes },
-    { 't', "test",            Arg_parser::no  },
-    { 'v', "verbose",         Arg_parser::no  },
-    { 'V', "version",         Arg_parser::no  },
-    {  0 , 0,                 Arg_parser::no  } };
+    { '0', "fast",              Arg_parser::no  },
+    { '1',  0,                  Arg_parser::no  },
+    { '2',  0,                  Arg_parser::no  },
+    { '3',  0,                  Arg_parser::no  },
+    { '4',  0,                  Arg_parser::no  },
+    { '5',  0,                  Arg_parser::no  },
+    { '6',  0,                  Arg_parser::no  },
+    { '7',  0,                  Arg_parser::no  },
+    { '8',  0,                  Arg_parser::no  },
+    { '9', "best",              Arg_parser::no  },
+    { 'a', "trailing-error",    Arg_parser::no  },
+    { 'b', "member-size",       Arg_parser::yes },
+    { 'c', "stdout",            Arg_parser::no  },
+    { 'd', "decompress",        Arg_parser::no  },
+    { 'f', "force",             Arg_parser::no  },
+    { 'F', "recompress",        Arg_parser::no  },
+    { 'h', "help",              Arg_parser::no  },
+    { 'k', "keep",              Arg_parser::no  },
+    { 'l', "list",              Arg_parser::no  },
+    { 'm', "match-length",      Arg_parser::yes },
+    { 'n', "threads",           Arg_parser::yes },
+    { 'o', "output",            Arg_parser::yes },
+    { 'q', "quiet",             Arg_parser::no  },
+    { 's', "dictionary-size",   Arg_parser::yes },
+    { 'S', "volume-size",       Arg_parser::yes },
+    { 't', "test",              Arg_parser::no  },
+    { 'v', "verbose",           Arg_parser::no  },
+    { 'V', "version",           Arg_parser::no  },
+    { opt_lt, "loose-trailing", Arg_parser::no  },
+    {  0 , 0,                   Arg_parser::no  } };
 
   const Arg_parser parser( argc, argv, options );
   if( parser.error().size() )				// bad option
@@ -746,8 +851,8 @@ int main( const int argc, const char * const argv[] )
     {
     const int code = parser.code( argind );
     if( !code ) break;					// no more options
-    const std::string & arg = parser.argument( argind );
-    const char * const ptr = arg.c_str();
+    const std::string & sarg = parser.argument( argind );
+    const char * const arg = sarg.c_str();
     switch( code )
       {
       case '0': case '1': case '2': case '3': case '4':
@@ -755,34 +860,47 @@ int main( const int argc, const char * const argv[] )
                 zero = ( code == '0' );
                 encoder_options = option_mapping[code-'0']; break;
       case 'a': ignore_trailing = false; break;
-      case 'b': member_size = getnum( ptr, 100000, max_member_size ); break;
+      case 'b': member_size = getnum( arg, 100000, max_member_size ); break;
       case 'c': to_stdout = true; break;
-      case 'd': program_mode = m_decompress; break;
+      case 'd': set_mode( program_mode, m_decompress ); break;
       case 'f': force = true; break;
       case 'F': recompress = true; break;
       case 'h': show_help(); return 0;
       case 'k': keep_input_files = true; break;
+      case 'l': set_mode( program_mode, m_list ); break;
       case 'm': encoder_options.match_len_limit =
-                  getnum( ptr, min_match_len_limit, max_match_len );
+                  getnum( arg, min_match_len_limit, max_match_len );
                 zero = false; break;
       case 'n': break;
-      case 'o': default_output_filename = arg; break;
+      case 'o': default_output_filename = sarg; break;
       case 'q': verbosity = -1; break;
-      case 's': encoder_options.dictionary_size = get_dict_size( ptr );
+      case 's': encoder_options.dictionary_size = get_dict_size( arg );
                 zero = false; break;
-      case 'S': volume_size = getnum( ptr, 100000, max_volume_size ); break;
-      case 't': program_mode = m_test; break;
+      case 'S': volume_size = getnum( arg, 100000, max_volume_size ); break;
+      case 't': set_mode( program_mode, m_test ); break;
       case 'v': if( verbosity < 4 ) ++verbosity; break;
       case 'V': show_version(); return 0;
+      case opt_lt: loose_trailing = true; break;
       default : internal_error( "uncaught option." );
       }
     } // end process options
 
-#if defined(__MSVCRT__) || defined(__OS2__)
+#if defined(__MSVCRT__) || defined(__OS2__) || defined(__DJGPP__)
   setmode( STDIN_FILENO, O_BINARY );
   setmode( STDOUT_FILENO, O_BINARY );
 #endif
 
+  bool filenames_given = false;
+  for( ; argind < parser.arguments(); ++argind )
+    {
+    filenames.push_back( parser.argument( argind ) );
+    if( filenames.back() != "-" ) filenames_given = true;
+    }
+  if( filenames.empty() ) filenames.push_back("-");
+
+  if( program_mode == m_list )
+    return list_files( filenames, ignore_trailing, loose_trailing );
+
   if( program_mode == m_test )
     outfd = -1;
   else if( program_mode == m_compress )
@@ -791,31 +909,25 @@ int main( const int argc, const char * const argv[] )
     prob_prices.init();
     }
 
-  bool filenames_given = false;
-  for( ; argind < parser.arguments(); ++argind )
-    {
-    filenames.push_back( parser.argument( argind ) );
-    if( filenames.back() != "-" ) filenames_given = true;
-    }
-
-  if( filenames.empty() ) filenames.push_back("-");
   if( !to_stdout && program_mode != m_test &&
       ( filenames_given || default_output_filename.size() ) )
-    set_signals();
+    set_signals( signal_handler );
 
-  Pretty_print pp( filenames, verbosity );
+  Pretty_print pp( filenames );
 
+  int failed_tests = 0;
   int retval = 0;
   bool stdin_used = false;
   for( unsigned i = 0; i < filenames.size(); ++i )
     {
+    std::string input_filename;
+    int infd;
     struct stat in_stats;
     output_filename.clear();
 
     if( filenames[i].empty() || filenames[i] == "-" )
       {
       if( stdin_used ) continue; else stdin_used = true;
-      input_filename.clear();
       infd = STDIN_FILENO;
       if( program_mode != m_test )
         {
@@ -824,12 +936,12 @@ int main( const int argc, const char * const argv[] )
         else
           {
           if( program_mode == m_compress )
-            set_c_outname( default_output_filename, volume_size > 0 );
+            set_c_outname( default_output_filename, false, volume_size > 0 );
           else output_filename = default_output_filename;
           if( !open_outstream( force, true ) )
             {
             if( retval < 1 ) retval = 1;
-            close( infd ); infd = -1;
+            close( infd );
             continue;
             }
           }
@@ -837,10 +949,9 @@ int main( const int argc, const char * const argv[] )
       }
     else
       {
-      input_filename = filenames[i];
-      const int eindex = extension_index( input_filename );
-      infd = open_instream( input_filename.c_str(), &in_stats, program_mode,
-                            eindex, recompress, to_stdout );
+      const int eindex = extension_index( input_filename = filenames[i] );
+      infd = open_instream2( input_filename.c_str(), &in_stats, program_mode,
+                             eindex, recompress, to_stdout );
       if( infd < 0 ) { if( retval < 1 ) retval = 1; continue; }
       if( program_mode != m_test )
         {
@@ -848,48 +959,65 @@ int main( const int argc, const char * const argv[] )
         else
           {
           if( program_mode == m_compress )
-            set_c_outname( input_filename, volume_size > 0 );
+            set_c_outname( input_filename, true, volume_size > 0 );
           else set_d_outname( input_filename, eindex );
           if( !open_outstream( force, false ) )
             {
             if( retval < 1 ) retval = 1;
-            close( infd ); infd = -1;
+            close( infd );
             continue;
             }
           }
         }
       }
 
-    if( !check_tty( infd, program_mode ) )
+    pp.set_name( input_filename );
+    if( !check_tty( pp.name(), infd, program_mode ) )
       {
       if( retval < 1 ) retval = 1;
+      if( program_mode == m_test ) { close( infd ); continue; }
       cleanup_and_fail( retval );
       }
 
     const struct stat * const in_statsp = input_filename.size() ? &in_stats : 0;
-    pp.set_name( input_filename );
+    const unsigned long long cfile_size =
+      ( in_statsp && S_ISREG( in_statsp->st_mode ) ) ?
+        ( in_statsp->st_size + 99 ) / 100 : 0;
     int tmp;
     if( program_mode == m_compress )
-      tmp = compress( member_size, volume_size, infd, encoder_options, pp,
-                      in_statsp, zero );
+      tmp = compress( cfile_size, member_size, volume_size, infd,
+                      encoder_options, pp, in_statsp, zero );
     else
-      tmp = decompress( infd, pp, ignore_trailing, program_mode == m_test );
+      tmp = decompress( cfile_size, infd, pp, ignore_trailing,
+                        loose_trailing, program_mode == m_test );
+    if( close( infd ) != 0 )
+      {
+      show_error( input_filename.size() ? "Error closing input file" :
+                                          "Error closing stdin", errno );
+      if( tmp < 1 ) tmp = 1;
+      }
     if( tmp > retval ) retval = tmp;
-    if( tmp && program_mode != m_test ) cleanup_and_fail( retval );
+    if( tmp )
+      { if( program_mode != m_test ) cleanup_and_fail( retval );
+        else ++failed_tests; }
 
     if( delete_output_on_interrupt )
       close_and_set_permissions( in_statsp );
     if( input_filename.size() )
       {
-      close( infd ); infd = -1;
-      if( !keep_input_files && !to_stdout && program_mode != m_test )
+      if( !keep_input_files && !to_stdout && program_mode != m_test &&
+          ( program_mode != m_compress || volume_size == 0 ) )
         std::remove( input_filename.c_str() );
       }
     }
   if( outfd >= 0 && close( outfd ) != 0 )
     {
-    show_error( "Can't close stdout", errno );
+    show_error( "Error closing stdout", errno );
     if( retval < 1 ) retval = 1;
     }
+  if( failed_tests > 0 && verbosity >= 1 && filenames.size() > 1 )
+    std::fprintf( stderr, "%s: warning: %d %s failed the test.\n",
+                  program_name, failed_tests,
+                  ( failed_tests == 1 ) ? "file" : "files" );
   return retval;
   }
